/**
 * Declares the axiom provider interface.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_AXIOM_PROVIDER_H
#define TME_AXIOM_PROVIDER_H

#include "tme_atomic.h"
#include "tme_vocabulary.h"

TME_BEGIN_DECLS

/// @cond
tme_struct_decl(TmeAxiomProvider);
tme_struct_decl(TmeSubsumers);
tme_struct_decl(TmeSemDesc);
/// @endcond

/**
 * This structure acts as an axiom provider for the reasoner.
 *
 * The TmeAxiomProvider interface must be implemented in order to provide axioms to the reasoner
 * instance. This allows the inference services implemented in the Core to be reusable
 * regardless of how axioms are stored or parsed.
 *
 * Implementation of this interface must come in the form of function pointers provided to the
 * tme_axiom_provider_alloc() function. Please refer to its documentation for further info.
 *
 * @see TmeReasoner
 *
 * @struct TmeAxiomProvider
 */

/**
 * Allocates a new axiom provider.
 *
 * Unless otherwise stated, the data structures returned by the functions implementing
 * the axiom provider interface must respect the following rules:
 *
 * - They must not be NULL.
 *
 * - They must be dynamically allocated.
 *
 * - They must never be directly freed.
 *
 * @param axiom_source Context for the subsequent handlers.
 * @param get_vocabulary Implements tme_axiom_provider_get_vocabulary().
 * @param get_atomic_concepts Implements tme_axiom_provider_get_concepts().
 * @param get_concept_description Implements tme_axiom_provider_get_concept_description().
 * @param get_atomic_individuals Implements tme_axiom_provider_get_individuals().
 * @param get_individual_description Implements tme_axiom_provider_get_individual_description().
 * @param axiom_source_free Called as part of tme_axiom_provider_free().
 * @return Axiom provider instance.
 *
 * @public @memberof TmeAxiomProvider
 */
TME_PUBLIC
TmeAxiomProvider* tme_axiom_provider_alloc
    (void *axiom_source,
     TmeVocabulary (*get_vocabulary)(void *axiom_source),
     Vector(TmeAtomicConcept)* (*get_atomic_concepts)(void *axiom_source),
     TmeSemDesc* (*get_concept_description)(void *axiom_source, TmeAtomicConcept concept,
                                            TmeSubsumers *subsumers),
     Vector(TmeAtomicIndividual)* (*get_atomic_individuals)(void *axiom_source),
     TmeSemDesc* (*get_individual_description)(void *axiom_source, TmeAtomicIndividual individual),
     void (*axiom_source_free)(void *axiom_source));

/**
 * Deallocates the axiom provider.
 *
 * @param provider Axiom provider to deallocate.
 *
 * @public @memberof TmeAxiomProvider
 */
TME_PUBLIC
void tme_axiom_provider_free(TmeAxiomProvider *provider);

/**
 * Retrieves the vocabulary.
 *
 * @param provider Axiom provider instance.
 * @return Vocabulary.
 *
 * @public @memberof TmeAxiomProvider
 */
TME_PUBLIC
TmeVocabulary tme_axiom_provider_get_vocabulary(TmeAxiomProvider const *provider);

/**
 * Retrieves all the concepts in the TBox.
 *
 * This function must return all the concepts present in the TBox. The reasoner instance
 * iterates over them, calling tme_axiom_provider_get_concept_description() for each concept,
 * in order to populate its data structures.
 *
 * @param provider Axiom provider instance.
 * @return All the concepts in the TBox.
 *
 * @public @memberof TmeAxiomProvider
 */
TME_PUBLIC
Vector(TmeAtomicConcept)* tme_axiom_provider_get_concepts(TmeAxiomProvider const *provider);

/**
 * Retrieves the semantic description of the specified concept.
 *
 * The returned description must be the intersection of subsumers and equivalents.
 * It is not necessary to intersect subsumers recursively: direct subsumers are enough.
 *
 * Furthermore, it is essential to populate the subsumers structure as follows:
 *
 * - Named equivalents must be marked as such via tme_subsumers_add_named_equivalence().
 *
 * - Named subsumers must be marked as such via tme_subsumers_add_told().
 *   Also in this case, direct subsumers are enough.
 *
 * - As a special case, subsumers of the bottom concept are actually equivalent to it, so
 *   they must be marked via tme_subsumers_add_named_equivalence().
 *
 * - The presence of an equivalence with a class expression must be marked by calling
 *   tme_subsumers_mark_equivalence_with_expression().
 *
 * @param provider Axiom provider instance.
 * @param concept Concept whose description should be retrieved.
 * @param [out] subsumers Subsumers of the specified concept.
 * @return Semantic description of the specified concept.
 *
 * @public @memberof TmeAxiomProvider
 */
TME_PUBLIC
TmeSemDesc* tme_axiom_provider_get_concept_description(TmeAxiomProvider const *provider,
                                                       TmeAtomicConcept concept,
                                                       TmeSubsumers *subsumers);

/**
 * Retrieves all the individuals in the ABox.
 *
 * This function must return all the individuals present in the ABox. The reasoner instance
 * iterates over them, calling tme_axiom_provider_get_individual_description() for each individual,
 * in order to populate its data structures.
 *
 * @param provider Axiom provider instance.
 * @return All the individuals in the ABox.
 *
 * @public @memberof TmeAxiomProvider
 */
TME_PUBLIC
Vector(TmeAtomicIndividual)* tme_axiom_provider_get_individuals(TmeAxiomProvider const *provider);

/**
 * Retrieves the semantic description of the specified individual.
 *
 * The returned description must be the intersection of the types associated with the individual.
 * It is not necessary to intersect types recursively: "leaf" types are enough.
 *
 * @param provider Axiom provider instance.
 * @param individual Individual whose description should be retrieved.
 * @return Semantic description of the specified individual.
 *
 * @public @memberof TmeAxiomProvider
 */
TME_PUBLIC
TmeSemDesc* tme_axiom_provider_get_individual_description(TmeAxiomProvider const *provider,
                                                          TmeAtomicIndividual individual);

TME_END_DECLS

#endif // TME_AXIOM_PROVIDER_H
