/**
 * Defines miscellaneous compatibility macros.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_COMPAT_H
#define TME_COMPAT_H

/**
 * Marks the beginning of declarations.
 *
 * @def TME_BEGIN_DECLS
 */

/**
 * Marks the end of declarations.
 *
 * @def TME_END_DECLS
 */

#ifdef __cplusplus
    #define TME_BEGIN_DECLS extern "C" {
    #define TME_END_DECLS }
#else
    #define TME_BEGIN_DECLS
    #define TME_END_DECLS
#endif

/**
 * Marks public API, whose symbols should be exported.
 *
 * @def TME_PUBLIC
 */

/**
 * Marks private API, whose symbols should not be exported.
 *
 * @def TME_PRIVATE
 */

#if defined _WIN32 || defined __CYGWIN__
    #ifdef TME_DLL
        #ifdef __GNUC__
            #define TME_PUBLIC __attribute__ ((dllexport))
        #else
            #define TME_PUBLIC __declspec(dllexport)
        #endif
    #else
        #ifdef __GNUC__
            #define TME_PUBLIC __attribute__ ((dllimport))
        #else
            #define TME_PUBLIC __declspec(dllimport)
        #endif
    #endif
    #define TME_PRIVATE
#else
    #if __GNUC__ >= 4
        #define TME_PUBLIC __attribute__ ((visibility ("default")))
        #define TME_PRIVATE  __attribute__ ((visibility ("hidden")))
    #else
        #define TME_PUBLIC
        #define TME_PRIVATE
    #endif
#endif

/// Suppresses unused variable warnings.
#ifndef tme_unused
    #if (defined __clang__ && __clang_major__ >= 3) || (defined __GNUC__ && __GNUC__ >= 3)
        #define tme_unused __attribute__((__unused__))
    #else
        #define tme_unused
    #endif
#endif

/**
 * Struct type.
 *
 * @param NAME Name of the struct type.
 */
#define tme_struct(NAME) struct NAME##_s

/**
 * Struct type forward declaration.
 *
 * @param NAME Name of the struct type.
 */
#define tme_struct_decl(NAME) typedef tme_struct(NAME) NAME

/**
 * Hash table type forward declaration.
 *
 * @param NAME Name of the hash table type.
 */
#define tme_hash_decl(NAME) typedef uhash_struct(NAME) UHash(NAME)

#endif // TME_COMPAT_H
