/**
 * Declares utility functions that generate string representations of known types.
 *
 * @note Returned strings must be freed via tme_string_deinit().
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_STRING_UTILS_H
#define TME_STRING_UTILS_H

#include "tme_atomic.h"
#include "tme_cardinality_role.h"
#include "tme_string.h"
#include "tme_universal_role.h"

TME_BEGIN_DECLS

/// @cond
tme_struct_decl(TmeAbduction);
tme_struct_decl(TmeBonus);
tme_struct_decl(TmeComposition);
tme_struct_decl(TmeContraction);
tme_struct_decl(TmeSemDesc);
tme_struct_decl(TmeStringProvider);
/// @endcond

/// Represents supported time units.
typedef enum TmeTimeUnit {

    /// Automatically determine the most appropriate time unit.
        MTU_AUTO,

    /// Nanosecond time unit.
        MTU_NANOS,

    /// Microsecond time unit.
        MTU_MICROS,

    /// Millisecond time unit.
        MTU_MILLIS,

    /// Second time unit.
        MTU_SECONDS

} TmeTimeUnit;

/// @name Model

/**
 * Returns the IRI associated with the specified entity.
 *
 * @param provider String provider instance.
 * @param entity_ptr Entity identifier.
 * @return String representation.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
TmeString tme_string_for_entity_iri(TmeStringProvider const *provider, TmeEntityPtr entity_ptr);

/**
 * Returns a readable representation of the specified entity.
 *
 * @param provider String provider instance.
 * @param entity_ptr Entity identifier.
 * @return String representation.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
TmeString tme_string_for_entity(TmeStringProvider const *provider, TmeEntityPtr entity_ptr);

/**
 * Returns a readable representation of the specified negated entity.
 *
 * @param provider String provider instance.
 * @param entity_ptr Entity identifier.
 * @return String representation.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
TmeString tme_string_for_negated_entity(TmeStringProvider const *provider, TmeEntityPtr entity_ptr);

/**
 * Returns a readable representation of the specified cardinality restriction.
 *
 * @param provider String provider instance.
 * @param role Cardinality restriction.
 * @return String representation.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
TmeString tme_string_for_card_role(TmeStringProvider const *provider, TmeCardinalityRole role);

/**
 * Returns a readable representation of the specified minimum cardinality restriction.
 *
 * @param provider String provider instance.
 * @param role Cardinality restriction.
 * @return String representation.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
TmeString tme_string_for_gt_role(TmeStringProvider const *provider, TmeCardinalityRole role);

/**
 * Returns a readable representation of the specified maximum cardinality restriction.
 *
 * @param provider String provider instance.
 * @param role Cardinality restriction.
 * @return String representation.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
TmeString tme_string_for_lt_role(TmeStringProvider const *provider, TmeCardinalityRole role);

/**
 * Returns a readable representation of the specified value restriction.
 *
 * @param provider String provider instance.
 * @param role Value restriction.
 * @return String representation.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
TmeString tme_string_for_uv_role(TmeStringProvider const *provider, TmeUniversalRole role);

/**
 * Returns a readable representation of the specified semantic description.
 *
 * @param provider String provider instance.
 * @param desc Semantic description.
 * @return String representation.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
TmeString tme_string_for_sem_desc(TmeStringProvider const *provider, TmeSemDesc const *desc);

/**
 * Returns a readable representation of the specified abduction result.
 *
 * @param provider String provider instance.
 * @param abduction Abduction result.
 * @return String representation.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
TmeString tme_string_for_abduction(TmeStringProvider const *provider, TmeAbduction abduction);

/**
 * Returns a readable representation of the specified bonus result.
 *
 * @param provider String provider instance.
 * @param bonus Bonus result.
 * @return String representation.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
TmeString tme_string_for_bonus(TmeStringProvider const *provider, TmeBonus bonus);

/**
 * Returns a readable representation of the specified contraction result.
 *
 * @param provider String provider instance.
 * @param contraction Contraction result.
 * @return String representation.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
TmeString tme_string_for_contraction(TmeStringProvider const *provider, TmeContraction contraction);

/**
 * Returns a readable representation of the specified covering result.
 *
 * @param provider String provider instance.
 * @param composition Covering result.
 * @return String representation.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
TmeString tme_string_for_composition(TmeStringProvider const *provider, TmeComposition composition);

/// @name Misc

/**
 * Returns a formatted, multi-line description of the reasoner (name, version, vendor, etc.).
 *
 * @return Reasoner description.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
TmeString tme_string_for_reasoner_metadata(void);

/**
 * Returns a readable representation of the specified time interval in nanoseconds.
 *
 * @param nanos Time interval in nanoseconds.
 * @param unit Desired time unit.
 * @return String representation.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
TmeString tme_string_from_nanos(uint64_t nanos, TmeTimeUnit unit);

/**
 * Sorts the specified entities by their human-readable string representation.
 *
 * @param provider String provider instance.
 * @param entities Entities to sort.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
void tme_string_sort_entities(TmeStringProvider *provider, Vector(TmeEntityPtr) *entities);

TME_END_DECLS

#endif // TME_STRING_UTILS_H
